<?php

namespace Drupal\cctags\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Database;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'cctags' block.
 *
 * @Block(
 *   id = "cctags_block",
 *   admin_label = @Translation("Cctags Block"),
 *   category = @Translation("Cctags")
 * )
 */
class CctagsBlock extends BlockBase implements ContainerFactoryPluginInterface {
  /**
   * Stores the configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['level'] = [
      '#type' => 'select',
      '#options' => _cctags_get_select_list('level'),
      '#title' => $this->t('Number of levels fonts metrics'),
      '#default_value' => $config['level'] ?? '',
      '#description' => $this->t('The number of levels between the least popular tags and the most popular ones. Different levels will be assigned a different class to be themed in cctags.css'),
    ];
    $form['tags'] = [
      '#type' => 'select',
      '#title' => 'Tags to show',
      '#options' => _cctags_get_select_list('numtags'),
      '#default_value' => $config['tags'] ?? '',
      '#maxlength' => 3,
      '#description' => $this->t('The number of tags to show in this block.'),
    ];
    $op_sort = ['level,asc' => $this->t('by level, ascending'), 'level,desc' => $this->t('by level, descending'), 'title,asc' => $this->t('by title, ascending'), 'title,desc' => $this->t('by title, descending'), 'random,none' => $this->t('random')];
    $form['tags_sort'] = [
      '#type' => 'radios',
      '#title' => $this->t('Tags sort order'),
      '#options' => $op_sort,
      '#default_value' => $config['tags_sort'] ?? '',
    ];
    $form['extra_class'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block wrapper extra class'),
      '#maxlength' => 64,
      '#description' => $this->t('Extra class for block wrapper.'),
      '#default_value' => $config['extra_class'] ?? '',
    ];

    $form['tags_more'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable more link of end block'),
      '#default_value' => $config['tags_more'] ?? FALSE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $connection = Database::getConnection();
    $config = $this->getConfiguration();

    $query = $connection->select('cctags', 'c');
    $query->fields('c', ['cctid']);
    if (isset($config['block_id'])) {
      $query->condition('block_id', $this->getConfiguration()['block_id']);
    }
    $cctid = $query->execute()->fetchField();

    $terms = cctags_get_level_tags($cctid);
    $content = [
      '#theme' => 'cctags_level',
      '#terms' => $terms,
    ];
    return [
      '#theme' => 'cctags_block',
      '#content' => $content,
    ];

  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    if ($account->isAnonymous()) {
      return AccessResult::forbidden();
    }
    return AccessResult::allowed();
  }

}
